/**
 * JASMINe
 * Copyright (C) 2005-2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JmxCnxWrapper.java 38 2007-12-12 13:03:04Z tokmensa $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.mbeancmd;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.LinkedHashSet;
import java.util.Set;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.IntrospectionException;
import javax.management.InvalidAttributeValueException;
import javax.management.ListenerNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServerConnection;
import javax.management.NotCompliantMBeanException;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.management.ObjectInstance;
import javax.management.ObjectName;
import javax.management.QueryExp;
import javax.management.ReflectionException;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

/**
 * JMX connection wrapper.
 */
public class JmxCnxWrapper implements MBeanServerConnection {
    /**
     * Constructor: saves the JMX URL.
     *
     * @param jmxUrl  JMX URL to use in the wrapper.
     */
    public JmxCnxWrapper(final String jmxUrl) {
        this.jmxUrl = jmxUrl;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#addNotificationListener(javax.management.ObjectName,
     *      javax.management.NotificationListener,
     *      javax.management.NotificationFilter, java.lang.Object)
     */
    public void addNotificationListener(ObjectName arg0,
            NotificationListener arg1, NotificationFilter arg2, Object arg3)
            throws InstanceNotFoundException, IOException {
        try {
            connect();
            mbscnx.addNotificationListener(arg0, arg1, arg2, arg3);
        } catch (IOException e) {
            analyseException(e);
        }
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#addNotificationListener(javax.management.ObjectName,
     *      javax.management.ObjectName, javax.management.NotificationFilter,
     *      java.lang.Object)
     */
    public void addNotificationListener(ObjectName arg0, ObjectName arg1,
            NotificationFilter arg2, Object arg3)
            throws InstanceNotFoundException, IOException {
        try {
            connect();
            mbscnx.addNotificationListener(arg0, arg1, arg2, arg3);
        } catch (IOException e) {
            analyseException(e);
        }
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#createMBean(java.lang.String,
     *      javax.management.ObjectName)
     */
    public ObjectInstance createMBean(String arg0, ObjectName arg1)
            throws ReflectionException, InstanceAlreadyExistsException,
            MBeanRegistrationException, MBeanException,
            NotCompliantMBeanException, IOException {
        ObjectInstance oi = null;
        try {
            connect();
            oi = mbscnx.createMBean(arg0, arg1);
        } catch (IOException e) {
            analyseException(e);
        }
        return oi;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#createMBean(java.lang.String,
     *      javax.management.ObjectName, javax.management.ObjectName)
     */
    public ObjectInstance createMBean(String arg0, ObjectName arg1,
            ObjectName arg2) throws ReflectionException,
            InstanceAlreadyExistsException, MBeanRegistrationException,
            MBeanException, NotCompliantMBeanException,
            InstanceNotFoundException, IOException {
        ObjectInstance oi = null;
        try {
            connect();
            oi = mbscnx.createMBean(arg0, arg1, arg2);
        } catch (IOException e) {
            analyseException(e);
        }
        return oi;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#createMBean(java.lang.String,
     *      javax.management.ObjectName, java.lang.Object[], java.lang.String[])
     */
    public ObjectInstance createMBean(String arg0, ObjectName arg1,
            Object[] arg2, String[] arg3) throws ReflectionException,
            InstanceAlreadyExistsException, MBeanRegistrationException,
            MBeanException, NotCompliantMBeanException, IOException {
        ObjectInstance oi = null;
        try {
            connect();
            oi = mbscnx.createMBean(arg0, arg1, arg2, arg3);
        } catch (IOException e) {
            analyseException(e);
        }
        return oi;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#createMBean(java.lang.String,
     *      javax.management.ObjectName, javax.management.ObjectName,
     *      java.lang.Object[], java.lang.String[])
     */
    public ObjectInstance createMBean(String arg0, ObjectName arg1,
            ObjectName arg2, Object[] arg3, String[] arg4)
            throws ReflectionException, InstanceAlreadyExistsException,
            MBeanRegistrationException, MBeanException,
            NotCompliantMBeanException, InstanceNotFoundException, IOException {
        ObjectInstance oi = null;
        try {
            connect();
            oi = mbscnx.createMBean(arg0, arg1, arg2, arg3, arg4);
        } catch (IOException e) {
            analyseException(e);
        }
        return oi;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#getAttribute(javax.management.ObjectName,
     *      java.lang.String)
     */
    public Object getAttribute(ObjectName arg0, String arg1)
            throws MBeanException, AttributeNotFoundException,
            InstanceNotFoundException, ReflectionException, IOException {
        Object att = null;
        try {
            connect();
            att = mbscnx.getAttribute(arg0, arg1);
        } catch (IOException e) {
            analyseException(e);
        }
        return att;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#getAttributes(javax.management.ObjectName,
     *      java.lang.String[])
     */
    public AttributeList getAttributes(ObjectName arg0, String[] arg1)
            throws InstanceNotFoundException, ReflectionException, IOException {
        AttributeList attl = new AttributeList();
        try {
            connect();
            attl = mbscnx.getAttributes(arg0, arg1);
        } catch (IOException e) {
            analyseException(e);
        }
        return attl;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#getDefaultDomain()
     */
    public String getDefaultDomain() throws IOException {
        String domain = null;
        try {
            connect();
            domain = mbscnx.getDefaultDomain();
        } catch (IOException e) {
            analyseException(e);
        }
        return domain;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#getDomains()
     */
    public String[] getDomains() throws IOException {
        String[] domains = null;
        try {
            connect();
            domains = mbscnx.getDomains();
        } catch (IOException e) {
            analyseException(e);
        }
        return domains;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#getMBeanCount()
     */
    public Integer getMBeanCount() throws IOException {
        Integer count = null;
        try {
            connect();
            count = mbscnx.getMBeanCount();
        } catch (IOException e) {
            analyseException(e);
        }
        return count;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#getMBeanInfo(javax.management.ObjectName)
     */
    public MBeanInfo getMBeanInfo(ObjectName arg0)
            throws InstanceNotFoundException, IntrospectionException,
            ReflectionException, IOException {
        MBeanInfo mbi = null;
        try {
            connect();
            mbi = mbscnx.getMBeanInfo(arg0);
        } catch (IOException e) {
            analyseException(e);
        }
        return mbi;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#getObjectInstance(javax.management.ObjectName)
     */
    public ObjectInstance getObjectInstance(ObjectName arg0)
            throws InstanceNotFoundException, IOException {
        ObjectInstance oi = null;
        try {
            connect();
            oi = mbscnx.getObjectInstance(arg0);
        } catch (IOException e) {
            analyseException(e);
        }
        return oi;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#invoke(javax.management.ObjectName,
     *      java.lang.String, java.lang.Object[], java.lang.String[])
     */
    public Object invoke(ObjectName arg0, String arg1, Object[] arg2,
            String[] arg3) throws InstanceNotFoundException, MBeanException,
            ReflectionException, IOException {
        Object o = null;
        try {
            connect();
            o = mbscnx.invoke(arg0, arg1, arg2, arg3);
        } catch (IOException e) {
            analyseException(e);
        }
        return o;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#isInstanceOf(javax.management.ObjectName,
     *      java.lang.String)
     */
    public boolean isInstanceOf(ObjectName arg0, String arg1)
            throws InstanceNotFoundException, IOException {
        boolean b = false;
        try {
            connect();
            b = mbscnx.isInstanceOf(arg0, arg1);
        } catch (IOException e) {
            analyseException(e);
        }
        return b;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#isRegistered(javax.management.ObjectName)
     */
    public boolean isRegistered(ObjectName arg0) throws IOException {
        boolean b = false;
        try {
            connect();
            b = mbscnx.isRegistered(arg0);
        } catch (IOException e) {
            analyseException(e);
        }
        return b;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#queryMBeans(javax.management.ObjectName,
     *      javax.management.QueryExp)
     */
    public Set queryMBeans(ObjectName arg0, QueryExp arg1) throws IOException {
        Set ons = null;
        try {
            connect();
            ons = mbscnx.queryMBeans(arg0, arg1);
        } catch (IOException e) {
            analyseException(e);
        }
        return (ons != null ? ons : new LinkedHashSet());
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#queryNames(javax.management.ObjectName,
     *      javax.management.QueryExp)
     */
    public Set queryNames(ObjectName arg0, QueryExp arg1) throws IOException {
        Set ons = null;
        try {
            connect();
            ons = mbscnx.queryNames(arg0, arg1);
        } catch (IOException e) {
            analyseException(e);
        }
        return (ons != null ? ons : new LinkedHashSet());
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#removeNotificationListener(javax.management.ObjectName,
     *      javax.management.NotificationListener)
     */
    public void removeNotificationListener(ObjectName arg0,
            NotificationListener arg1) throws InstanceNotFoundException,
            ListenerNotFoundException, IOException {
        try {
            connect();
            mbscnx.removeNotificationListener(arg0, arg1);
        } catch (IOException e) {
            analyseException(e);
        }
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#removeNotificationListener(javax.management.ObjectName,
     *      javax.management.ObjectName)
     */
    public void removeNotificationListener(ObjectName arg0, ObjectName arg1)
            throws InstanceNotFoundException, ListenerNotFoundException,
            IOException {
        try {
            connect();
                mbscnx.removeNotificationListener(arg0, arg1);
        } catch (IOException e) {
            analyseException(e);
        }
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#removeNotificationListener(javax.management.ObjectName,
     *      javax.management.NotificationListener,
     *      javax.management.NotificationFilter, java.lang.Object)
     */
    public void removeNotificationListener(ObjectName arg0,
            NotificationListener arg1, NotificationFilter arg2, Object arg3)
            throws InstanceNotFoundException, ListenerNotFoundException,
            IOException {
        try {
            connect();
            mbscnx.removeNotificationListener(arg0, arg1, arg2, arg3);
        } catch (IOException e) {
            analyseException(e);
        }
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#removeNotificationListener(javax.management.ObjectName,
     *      javax.management.ObjectName, javax.management.NotificationFilter,
     *      java.lang.Object)
     */
    public void removeNotificationListener(ObjectName arg0, ObjectName arg1,
            NotificationFilter arg2, Object arg3)
            throws InstanceNotFoundException, ListenerNotFoundException,
            IOException {
        try {
            connect();
            mbscnx.removeNotificationListener(arg0, arg1, arg2, arg3);
        } catch (IOException e) {
            analyseException(e);
        }
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#setAttribute(javax.management.ObjectName,
     *      javax.management.Attribute)
     */
    public void setAttribute(ObjectName arg0, Attribute arg1)
            throws InstanceNotFoundException, AttributeNotFoundException,
            InvalidAttributeValueException, MBeanException,
            ReflectionException, IOException {
        try {
            connect();
            mbscnx.setAttribute(arg0, arg1);
        } catch (IOException e) {
            analyseException(e);
        }
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#setAttributes(javax.management.ObjectName,
     *      javax.management.AttributeList)
     */
    public AttributeList setAttributes(ObjectName arg0, AttributeList arg1)
            throws InstanceNotFoundException, ReflectionException, IOException {

        AttributeList attl = new AttributeList();
        try {
            connect();
            attl = mbscnx.setAttributes(arg0, arg1);
        } catch (IOException e) {
            analyseException(e);
        }
        return attl;
    }

    /**
     * Implementation of inherited method.
     *
     * @see javax.management.MBeanServerConnection#unregisterMBean(javax.management.ObjectName)
     */
    public void unregisterMBean(ObjectName arg0)
            throws InstanceNotFoundException, MBeanRegistrationException,
            IOException {
        try {
            connect();
            mbscnx.unregisterMBean(arg0);
        } catch (IOException e) {
            analyseException(e);
        }
    }

    /**
     * @param cnx  setMBeanServerConnection to use in the JMX connector.
     */
    public void setMBeanServerConnection(final MBeanServerConnection cnx) {
        mbscnx = cnx;
    }

    /**
     * @return  The JMX URL used by this JMX connector.
     */
    public String getJmxUrl() {
        return jmxUrl;
    }

    /**
     * @param url  The JMX URL to be used by this JMX connector.
     */
    public void setJmxUrl(final String url) {
        jmxUrl = url;
    }

    /**
     * Connects to the JMX URL.
     *
     * @throws IOException  If connection failed.
     */
    public void connect() throws IOException {
        if (mbscnx == null) {
            JMXConnector c = null;
            JMXServiceURL url = new JMXServiceURL(jmxUrl);
            try {
                c = JMXConnectorFactory.connect(url);
                mbscnx = c.getMBeanServerConnection();
                jmxConnector = c;
            } catch (MalformedURLException e) {
                throw e;
            }
        }
    }

    /**
     * Tests the JmxCnxWrapper implementation.
     *
     * @param args  URL to use as first argument.
     */
    public static void main(final String[] args) {

        // bad name
        // String url = "service:jmx:rmi:///jndi/rmi:///jrmpconnector_jonas";

        // wrong port or host
        // String url =
        // "service:jmx:rmi:///jndi/rmi://localhost:2345/jrmpconnector_jonas";

        // unresoved host
        // String url =
        // "service:jmx:rmi:///jndi/rmi://total:2345/jrmpconnector_jonas";

        // wrong address
        // String url =
        // "service:jmx:rmi:///jndi/rmi://192.168.1.111:2345/jrmpconnector_jonas";

        // right addres
        String url = "service:jmx:rmi:///jndi/rmi://localhost:1099/jrmpconnector_j486_A";

        if (args.length != 0) {
            url = args[0];
        }
        JmxCnxWrapper cnx = new JmxCnxWrapper(url);

        for (int i = 1; i <= 100; i++) {
            try {
                Integer cnt = cnx.getMBeanCount();
                System.out.println("COUNT= " + cnt);
            } catch (IOException e) {
                System.err.println("FWA : " + e.getMessage());
            }
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

    }

    /**
     * Analyses e, closes JMX if necessary and and throws e back.
     *
     * @param e  Exception to analyse.
     *
     * @throws IOException  e.
     */
    private void analyseException(final IOException e) throws IOException {
        boolean toClose = false;
        if (e instanceof java.rmi.ConnectException) {
            toClose = true;
        } else if (e instanceof java.rmi.ConnectIOException) {
            toClose = true;
        } else if (e instanceof java.rmi.UnknownHostException) {
            toClose = true;
        } else if (e instanceof java.rmi.RemoteException) {
            // So what ?
        } else {
            String s = e.getMessage();
            if (s.startsWith("javax.naming.NameNotFoundException")) {
                System.err.println("Persistent error : " + e.getMessage());
            } else if (s.startsWith("javax.naming.ServiceUnavailableException")) {
                // ex: java.rmi.ConnectException: Connection refused to host
                toClose = true;
            } else if (s.startsWith("javax.naming.ConfigurationException")) {
                // ex: java.rmi.UnknownHostException
                toClose = true;
            } else {
                // Nothing to do
            }
        }
        if (toClose) {
            closeJMX();
        }
        throw e;
    }

    /**
     * Closes the JMX connection.
     */
    private void closeJMX() {
        if (jmxConnector != null) {
            try {
                jmxConnector.close();
            } catch (IOException e) {
                System.err.println("FWA closing connector)");
                e.printStackTrace(System.err);
            }
        }
        mbscnx = null;
    }

    /**
     * Connection to the MBean server.
     */
    private MBeanServerConnection mbscnx = null;

    /**
     * JMX connector.
     */
    private JMXConnector jmxConnector = null;

    /**
     * JMX URL to connect to.
     */
    private String jmxUrl = null;
}
