/**
 * JASMINe
 * Copyright (C) 2005-2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MonologJMX.java 38 2007-12-12 13:03:04Z tokmensa $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.mbeancmd.commands;

import java.io.IOException;
import java.util.Date;
import java.util.logging.LogRecord;

import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.NotCompliantMBeanException;
import javax.management.Notification;
import javax.management.NotificationListener;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.loading.MLet;

import org.ow2.jasmine.monitoring.mbeancmd.JmxAP;

/**
 * Command that displays all Monolog logs from a given J2EE server.
 *
 * TODO: Make this class inherit AbstractCommand and able of receiving command
 *       line arguments (output to a file, for instance?).
 */
public class MonologJMX {
    /**
     * Constructor: saves the JMX URL.
     *
     * @param jmxUrl  Target JMX URL.
     */
    public MonologJMX(final String jmxUrl) {
        this.jmxUrl = jmxUrl;
    }

    /**
     * @return  JMX connection to the MBean server.
     */
    protected MBeanServerConnection getMBeanServerConnection(){
        if (mbscnx != null) {
            return mbscnx;
        }

        try {
            if (jmxUrl == null) {
                mbscnx = (new JmxAP()).getMBeanServerConnection();
            } else {
                mbscnx = (new JmxAP(jmxUrl)).getMBeanServerConnection();
            }
        } catch (Exception ex){
            ex.printStackTrace(System.err);
        }
        return mbscnx;
    }

    /**
     * Displays all Monolog logs from a given J2EE server.
     *
     * @param args  Array of argument, first if the JMX URL of the J2EE server.
     */
    public static void main(final String[] args) {

        String url = "service:jmx:rmi:///jndi/rmi://localhost:1099/jrmpconnector_jonas";

        if (args.length > 0) {
            url = args[0];
        }

        MonologJMX mlog = new MonologJMX(url);

        MBeanServerConnection cnx = mlog.getMBeanServerConnection();

        NotificationListener listener = new NotificationListener() {
            public void handleNotification(final Notification nf, final Object arg1) {
                LogRecord record = (LogRecord) nf.getUserData();

                System.out.println("\n***   RECORD :");
                System.out.println("timestamp: " + new Date(record.getMillis()));
                System.out.println("stamp: " + record.getMillis());
                System.out.println("sequence: " + record.getSequenceNumber());
                System.out.println("loggerName: " + record.getLoggerName());
                System.out.println("level: " + record.getLevel());
                System.out.println("message: " + record.getMessage());
                System.out.println("source class: " + record.getSourceClassName());
                System.out.println("source method: " + record.getSourceMethodName());
                System.out.println("resource bundle: " + record.getResourceBundleName());

                String ty = nf.getType();
                System.out.println("nf.type: " + ty);
                System.out.println("nf.stamp: " + nf.getTimeStamp());
                System.out.println("nf.source: " + nf.getSource());
                System.out.println("nf.seq: " + nf.getSequenceNumber());
                System.out.println("nf.msq: " + nf.getMessage());
                System.out.println("End of record");
            }
        };

        ObjectName on;
        try {
            on = ObjectName.getInstance("fwa:type=service,name=log,fname=trace");
            cnx.addNotificationListener(on, listener, null, null);
        } catch (MalformedObjectNameException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (InstanceNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        try {
            Thread.sleep(5000);
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        // Now register a remote MBean, for example an MLet, so that the MBeanServerDelegate
        // will emit notifications for its registration
        ObjectName name;
        try {
            name = ObjectName.getInstance("jonas:type=mlet,name=MX4J");
             // First notification
              cnx.createMBean(MLet.class.getName(), name, null);
              // Second notification
              cnx.unregisterMBean(name);
        } catch (MalformedObjectNameException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (InstanceAlreadyExistsException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (MBeanRegistrationException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (NotCompliantMBeanException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (InstanceNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (ReflectionException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (MBeanException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        System.out.println("Bye...");
    }

    /**
     * JMX connection.
     */
    private MBeanServerConnection mbscnx = null;

    /**
     * JMX URL to connect to.
     */
    private String jmxUrl = null;
}
