/**
 * JASMINe
 * Copyright (C) 2005-2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Configurator.java 39 2007-12-12 14:51:20Z tokmensa $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.mbeancmd.graph.conf;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.TreeMap;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.dom4j.util.XMLErrorHandler;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Reads the configuration from the XML configuration file.
 */
public class Configurator implements Constants, Configuration {
    /**
     * Tests the Configurator implementation.
     *
     * @param args  Ignored.
     */
    public static void main(final String[] args) {

        Configuration cf = new Configurator();
        cf.loadConfig("./src/xml/graph2.xml");
        String[] s = (String[]) cf.getSerieIds();
        String[] g = (String[]) cf.getGraphIds();
        GraphConfig gc = cf.getGraphConfig("g1");
        s = gc.getSerieIds();
        System.out.println(g[0]);
        System.out.println(gc.getLegend(s[1]));
        System.out.println(cf.getSerieConfig(s[1]).getTitle());
        System.out.println("Absciss=" + cf.getAbscissaColumn());
        System.out.println("separator=" + cf.getSeparator());
        System.out.println("dateFormat=" + cf.getDateFormat().toPattern());
    }

    /**
     * Implementation of inherited method.
     *
     * @see Configuration#getSerieConfig
     */
    public SerieConfig getSerieConfig(final String id) {
        return series.get(id);
    }

    /**
     * Implementation of inherited method.
     *
     * @see Configuration#getSerieIds
     */
    public String[] getSerieIds() {
        String[] s = new String[1];
        return series.keySet().toArray(s);
    }

    /**
     * Implementation of inherited method.
     *
     * @see Configuration#getGraphConfig
     */
    public GraphConfig getGraphConfig(final String id) {
        return graphs.get(id);
    }

    /**
     * Implementation of inherited method.
     *
     * @see Configuration#getGraphIds
     */
    public String[] getGraphIds() {
        String[] s = new String[1];
        return graphs.keySet().toArray(s);
    }

    /**
     * Implementation of inherited method.
     *
     * @see Configuration#loadConfig
     */
    public void loadConfig(final String conf) {
        String configuration;
        if (conf == null) {
            configuration = System.getProperty(GRAPH_CONFIG_PATH);
        } else {
            configuration = conf;
        }

        SAXReader reader = new SAXReader();
        reader.setValidation(true);
        reader.setStripWhitespaceText(false);
        reader.setEntityResolver(new MyEntityResolver());

        // add error handler which turns any errors into XML
        reader.setErrorHandler(new XMLErrorHandler());

        // parse the document
        try {
            doc = reader.read(configuration);
        } catch (DocumentException e) {
            e.printStackTrace();
        }

        // build configration data
        parseCommon();
        parseSeries();
        parseGraphs();
    }

    /**
     * @return  The line number reader for the XML DTD.
     */
    public static LineNumberReader getDTD() {
        InputStream dtd= Thread.currentThread().getContextClassLoader().getResourceAsStream("xml/graph2.dtd");
        return new LineNumberReader(new InputStreamReader(dtd));
    }

    /**
     * Implementation of inherited method.
     *
     * @see Configuration#getSeparator
     */
    public String getSeparator() {
        return separator;
    }

    /**
     * Implementation of inherited method.
     *
     * @see Configuration#getAbscissaColumn
     */
    public String getAbscissaColumn() {
        return this.abscissaColumn;
    }

    /**
     * Implementation of inherited method.
     *
     * @see Configuration#getDateFormat
     */
    public SimpleDateFormat getDateFormat() {
        return simpleDateFormat;
    }

    /**
     * Implementation of inherited method.
     *
     * @see Configuration#isDateLong
     */
    public boolean isDateLong() {
        return (simpleDateFormat == null);
    }

    /**
     * Entity resolver.
     */
    private class MyEntityResolver implements EntityResolver {
        /**
         * Implementation of inherited method.
         *
         * @see EntityResolver#resolveEntity
         */
        public InputSource resolveEntity(final String publicId, final String systemId)
                throws SAXException, IOException {
            InputSource source = null;
            if (publicId.equals("-//Bull SAS//DTD JOnAS Monitor 1.0//EN")) {
                URL dtd= Thread.currentThread().getContextClassLoader().getResource("xml/graph2.dtd");
                source = new InputSource(dtd.toString());
            }
            return source;
        }
    }

    /**
     * Parses the series based on the XML configuration.
     */
    private void parseSeries() {
        List l = doc.selectNodes("//serie-def");
        Iterator it = l.iterator();
        while (it.hasNext()) {
            addSerie((Node) it.next());
        }
    }

    /**
     * Adds a series.
     *
     * @param node  XML node describing the series.
     */
    private void addSerie(final Node node) {
        SerieConfig sc = new SerieConfig(this);

        // ID
        String id = node.selectSingleNode("@id").getText();
        sc.setId(id);

        // title
        sc.setTitle(node.selectSingleNode("./name").getText());

        // description
        Node nd = node.selectSingleNode("./description");
        if (nd != null) {
            sc.setDescription(nd.getText());
        }

        // mbean
        nd = node.selectSingleNode("./pattern");
        if (nd != null) {
            sc.setMbeanPattern(nd.getText());
        }

        sc.setXAxis(abscissaColumn);
        sc.setXType(DataType.TIME);
        sc.setMbeanColumn(mbeanColumn);

        // y axis
        sc.setYAxis(node.selectSingleNode("./y/@col").getText());
        nd = node.selectSingleNode("./y/@type");
        if (nd != null) {
            String type = nd.getText();
            sc.setYType(DataType.valueOf(type.toUpperCase(Locale.ENGLISH)));
        }

        series.put(id, sc);
    }

    /**
     * Parses the graph based on the XML configuration.
     */
    private void parseGraphs() {
        List l = doc.selectNodes("//graph");
        Iterator it = l.iterator();
        while (it.hasNext()) {
            addGraph((Node) it.next());
        }
    }

    /**
     * Adds a graph.
     *
     * @param node  XML node describing the graph.
     */
    private void addGraph(final Node node) {
        GraphConfig gc = new GraphConfig(this);

        // ID
        String id = node.selectSingleNode("@id").getText();
        gc.setId(id);

        // title
        gc.setTitle(node.selectSingleNode("./title").getText());

        // series
        List ls = node.selectNodes("./serie");

        Iterator it = ls.iterator();

        while (it.hasNext()) {
            Node nd = (Node) it.next();
            String sid = nd.selectSingleNode("@ref").getText();
            String legend = parseSerieLegend(nd);
            gc.addSerie(sid, legend);
        }

        graphs.put(id, gc);
    }

    /**
     * Parses a legend.
     *
     * @param serie  XML node describing the legend.
     *
     * @return Parsed legend.
     */
    private String parseSerieLegend(final Node serie) {
        String serieId = serie.selectSingleNode("@ref").getText();
        String val = "";

        if (!serie.hasContent()) {
            // the legend is the name of the series
            val = doc.selectSingleNode( "//serie-def[@id='" + serieId +"']/name" ).getText();
        } else {
            // Agregate evaluated content
            Element element = (Element) serie;
            for (int i = 0, size = element.nodeCount(); i < size; i++) {
                Node n = element.node(i);
                String type = n.getName();
                if ("i".equals(type)) {
                    //n = n.selectSingleNode("./@link");
                    val += evalINode(n, serieId);
                } else {
                    val += n.getText();
                }
            }
        }

        // Remove extra blank characters
        val = val.replaceAll("[\\t\\n\\f\\r]", " ");
        val = val.trim();
        val = val.replaceAll("[ ]{2,}", " ");

        return val;
    }

    /**
     * Evaluate a node. It may contain:
     *
     *      - A ref: then it evaluates to the value associated to the ref.
     *      - A link: it is a XPath link, it evaluates to the value of the
     *                pointed to element.
     *      - A text: it evaluates to the contents of the element in
     *                the serie-def.
     *
     * If several attributes are present, the evaluation order is:
     *      1. text
     *      2. ref
     *      3. link
     *
     * @param inode     XML node.
     * @param serieId  Series ID corresponding to the node.
     *
     * @return Evaluated inode.
     */
    private String evalINode(final Node inode, final String serieId) {
        String value = "";
        boolean evaluated = false;

        // First look for the 'text' attribute
        Node node = inode.selectSingleNode("./@text");
        if (node != null) {
            String elementName = node.getStringValue();
            Node n = doc.selectSingleNode("//serie-def[@id='" + serieId +"']/"+ elementName);
            if (n != null) {
                value = n.getText();
                evaluated = true;
            }
        }

        // Then look for the ref attribute
        node = inode.selectSingleNode("./@ref");
        if ((!evaluated) && (node != null)) {
            Node n = doc.selectSingleNode("//*[@id='"+ node.getStringValue()+"']");
            if (n != null) {
                value = n.getText();
                evaluated = true;
            }
        }

        // Finally look for the link attribute
        node = inode.selectSingleNode("./@link");
        if ((!evaluated) && (node != null)) {
            Node n = doc.selectSingleNode(node.getStringValue());
            if (node != null) {
                value = n.getText();
                evaluated = true;
            }
        }

        return value;
    }

    /**
     * Calls all other parsers.
     */
    private void parseCommon() {

        Node node = doc.getRootElement();

        Node nd = node.selectSingleNode("./@separator");
        if (nd != null) {
            separator = nd.getText();
        }

        nd = node.selectSingleNode("./@absciss");
        if (nd != null) {
            abscissaColumn = nd.getText();
        }

        nd = node.selectSingleNode("./mbean");
        if (nd != null) {
            mbeanColumn = nd.getText();
        }

        nd = node.selectSingleNode("./@timeformat");
        if (nd != null) {
            String df = nd.getText();
            if (Constants.DATEFORMAT_LONG.equals(df)) {
                simpleDateFormat = null;
            } else {
                simpleDateFormat = new SimpleDateFormat(nd.getText());
            }
        } else {
            simpleDateFormat = new SimpleDateFormat(DEFAULT_DATE_FORMAT);
        }

    }

    /**
     * Map of series.
     */
    private Map<String, SerieConfig> series = new TreeMap<String, SerieConfig>();

    /**
     * Map of graphs.
     */
    private Map<String, GraphConfig> graphs = new TreeMap<String, GraphConfig>();

    /**
     * XML document.
     */
    private Document doc = null;

    /**
     * Field separator.
     */
    private String separator = DEFAULT_SEPARATOR;

    /**
     * Column used as abscissa.
     */
    private String abscissaColumn = DEFAULT_ABSCISSA_COLUMN;

    /**
     * Column used for the legend.
     */
    private String mbeanColumn = DEFAULT_MBEAN_COLUMN;

    /**
     * Date format.
     */
    private SimpleDateFormat simpleDateFormat = null;
}
